@@ -40,7 +40,11 @@
 
 #include "mymoneyfile.h"
 
-CsvWriter::CsvWriter()
+CsvWriter::CsvWriter() :
+    m_plugin(0),
+    m_firstSplit(false),
+    m_highestSplitCount(0),
+    m_noError(true)
 {
 }
 
@@ -98,7 +102,7 @@ void CsvWriter::writeAccountEntry(QTextStream& stream, const QString& accountId,
   QString type = account.accountTypeToString(account.accountType());
   data = QString(i18n("Account Type:"));
 
-  if (type == QLatin1String("Investment")) {
+  if (account.accountType() == MyMoneyAccount::Investment) {
     data += QString("%1\n\n").arg(type);
     m_headerLine << QString(i18n("Date")) << QString(i18n("Security")) << QString(i18n("Action/Type")) << QString(i18n("Amount")) << QString(i18n("Quantity")) << QString(i18n("Price")) << QString(i18n("Interest")) << QString(i18n("Fees")) << QString(i18n("Account")) << QString(i18n("Memo")) << QString(i18n("Status"));
     data += m_headerLine.join(m_separator);
@@ -153,16 +157,15 @@ void CsvWriter::writeCategoryEntries(QTextStream &s)
 void CsvWriter::writeCategoryEntry(QTextStream &s, const QString& accountId, const QString& leadIn)
 {
   MyMoneyAccount acc = MyMoneyFile::instance()->account(accountId);
-  QString name = acc.name();
+  QString name = format(acc.name());
 
-  s << leadIn << name << m_separator;
+  s << leadIn << name;
   s << (acc.accountGroup() == MyMoneyAccount::Expense ? QLatin1Char('E') : QLatin1Char('I'));
   s << endl;
 
   QStringList list = acc.accountList();
   QStringList::Iterator it_catList;
 
-  name += m_separator;
   for (it_catList = list.begin(); it_catList != list.end(); ++it_catList) {
     writeCategoryEntry(s, *it_catList, name);
   }
@@ -189,24 +192,15 @@ void CsvWriter::writeTransactionEntry(const MyMoneyTransaction& t, const QString
 
   str += QString("%1" + m_separator).arg(t.postDate().toString(Qt::ISODate));
   MyMoneyPayee payee = file->payee(split.payeeId());
-  str += QString("%1" + m_separator).arg(payee.name());
+  str += format(payee.name());
 
-  QString txt = split.value().formatMoney("", 2, false);
-  str += QString("%1" + m_separator).arg(txt);
+  str += format(split.value());
 
   if (splits.count() > 1) {
     MyMoneySplit sp = t.splitByAccount(accountId, false);
-    QString tmp = QString("%1").arg(file->accountToCategory(sp.accountId()));
-    str += tmp + m_separator;
+    str += format(file->accountToCategory(sp.accountId()));
   }
-  QString memo = split.memo();
-  memo.replace('\n', '~').remove('\'');
-  QString localeThousands = KGlobal::locale()->thousandsSeparator();  //  In case of clash with field separator
-  if (m_separator == localeThousands) {
-    memo.replace(localeThousands, QString());
-  }
-
-  str += QString("%1" + m_separator).arg(memo);
+  str += format(split.memo());
 
   switch (split.reconcileFlag()) {
     case MyMoneySplit::Cleared:
@@ -223,13 +217,13 @@ void CsvWriter::writeTransactionEntry(const MyMoneyTransaction& t, const QString
       break;
   }
 
-  str += split.number();
+  str += format(split.number(), false);
 
   if (splits.count() > 2) {
     QList<MyMoneySplit>::ConstIterator it;
     for (it = splits.constBegin(); it != splits.constEnd(); ++it) {
       if (!((*it) == split)) {
-        writeSplitEntry(str, *it, splits.count() - 1);
+        writeSplitEntry(str, *it, splits.count() - 1, it+1 == splits.constEnd());
       }
     }
   }
@@ -237,31 +231,23 @@ void CsvWriter::writeTransactionEntry(const MyMoneyTransaction& t, const QString
   m_map.insertMulti(date, str);
 }
 
-void CsvWriter::writeSplitEntry(QString &str, const MyMoneySplit& split, const int splitCount)
+void CsvWriter::writeSplitEntry(QString &str, const MyMoneySplit& split, const int splitCount, const int lastEntry)
 {
   if (m_firstSplit) {
     m_firstSplit = false;
     str += m_separator;
   }
   MyMoneyFile* file = MyMoneyFile::instance();
-  QString splt = QString("%1").arg(file->accountToCategory(split.accountId()));
-  str += splt + m_separator;
+  str += format(file->accountToCategory(split.accountId()));
 
   if (splitCount > m_highestSplitCount) {
     m_highestSplitCount++;
     m_headerLine << QString(i18n("splitCategory")) << QString(i18n("splitMemo")) << QString(i18n("splitAmount"));
     m_headerLine.join(m_separator);
   }
-  QString m = split.memo();
-  m.replace(QLatin1Char('\n'), QLatin1Char('~'));
-  QString localeThousands = KGlobal::locale()->thousandsSeparator();  //  In case of clash with field separator
-  if (m_separator == localeThousands) {
-    m.replace(localeThousands, QString());
-  }
-  str += QString("%1" + m_separator).arg(m);
+  str += format(split.memo());
 
-  QString txt = QString("%1" + m_separator).arg(split.value().formatMoney("", 2, false));
-  str += txt;
+  str += format(split.value(), 2, !lastEntry);
 }
 
 void CsvWriter::extractInvestmentEntries(const QString& accountId, const QDate& startDate, const QDate& endDate)
@@ -327,14 +313,12 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
           strStatus.clear();
           break;
       }
-      strStatus += m_separator;
     }
   }
   //
   //  Add date.
   //
   QString str = QString("\n%1" + m_separator).arg(t.postDate().toString(Qt::ISODate));
-  QString localeThousands = KGlobal::locale()->thousandsSeparator();  //  In case of clash with field separator
   for (itSplit = lst.begin(); itSplit != lst.end(); ++itSplit) {
     MyMoneyAccount acc = file->account((*itSplit).accountId());
     //
@@ -343,34 +327,33 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
     if ((acc.accountType() == MyMoneyAccount::Checkings) || (acc.accountType() == MyMoneyAccount::Cash) || (acc.accountType() == MyMoneyAccount::Savings)) {
       chkAccntId = (*itSplit).accountId();
       chkAccnt = file->account(chkAccntId).name();
-      strCheckingAccountName = file->accountToCategory(chkAccntId) + m_separator;
-      strAmount = (*itSplit).value().formatMoney("", 2).remove(localeThousands) + m_separator;
+      strCheckingAccountName = format(file->accountToCategory(chkAccntId));
+      strAmount = format((*itSplit).value());
     } else if (acc.accountType() == MyMoneyAccount::Income) {
       //
       //  MyMoneyAccount::Income.
       //
       qty = (*itSplit).shares();
       value = (*itSplit).value();
-      strInterest = value.formatMoney("", 2, false) + m_separator;
+      strInterest = format(value);
     } else if (acc.accountType() == MyMoneyAccount::Expense) {
       //
       //  MyMoneyAccount::Expense.
       //
       qty = (*itSplit).shares();
       value = (*itSplit).value();
-      strFees = value.formatMoney("", 2, false) + m_separator;
+      strFees = format(value);
     }  else if (acc.accountType() == MyMoneyAccount::Stock) {
       //
       //  MyMoneyAccount::Stock.
       //
-      strMemo = QString("%1" + m_separator).arg((*itSplit).memo());
-      strMemo.replace(QLatin1Char('\n'), QLatin1Char('~')).remove('\'');
+      strMemo = format((*itSplit).memo());
       //
       //  Actions.
       //
-      if ((*itSplit).action() == QLatin1String("Dividend")) {
+      if ((*itSplit).action() == MyMoneySplit::ActionDividend) {
         strAction = QLatin1String("DivX");
-      } else if ((*itSplit).action() == QLatin1String("IntIncome")) {
+      } else if ((*itSplit).action() == MyMoneySplit::ActionInterestIncome) {
         strAction = QLatin1String("IntIncX");
       }
       if ((strAction == QLatin1String("DivX")) || (strAction == QLatin1String("IntIncX"))) {
@@ -380,23 +363,23 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
                              i18n("Invalid transaction"));
           return;
         }
-      } else if ((*itSplit).action() == QLatin1String("Buy")) {
+      } else if ((*itSplit).action() == MyMoneySplit::ActionBuyShares) {
         qty = (*itSplit).shares();
         if (qty.isNegative()) {
           strAction = QLatin1String("Sell");
         } else {
           strAction = QLatin1String("Buy");
         }
-      } else if ((*itSplit).action() == QLatin1String("Add")) {
+      } else if ((*itSplit).action() == MyMoneySplit::ActionAddShares) {
         qty = (*itSplit).shares();
         if (qty.isNegative()) {
           strAction = QLatin1String("Shrsout");
         } else {
           strAction = QLatin1String("Shrsin");
         }
-      } else if ((*itSplit).action() == QLatin1String("Reinvest")) {
+      } else if ((*itSplit).action() == MyMoneySplit::ActionReinvestDividend) {
         qty = (*itSplit).shares();
-        strAmount = (*itSplit).value().formatMoney("", 2).remove(localeThousands) + m_separator;
+        strAmount = format((*itSplit).value());
         strAction = QLatin1String("ReinvDiv");
       } else {
         strAction = (*itSplit).action();
@@ -407,14 +390,14 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
       if ((strAction == QLatin1String("Buy")) || (strAction == QLatin1String("Sell")) || (strAction == QLatin1String("ReinvDiv"))) {
         //
         //  Add total.
-        //
+        // TODO: value is not used below
         if (strAction == QLatin1String("Sell")) {
           value = -value;
         }
         //
         //  Add price.
         //
-        strPrice = (*itSplit).price().formatMoney("", 6, false);
+        strPrice = format((*itSplit).price(), 6);
         if (!qty.isZero()) {
           //
           //  Add quantity.
@@ -422,7 +405,7 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
           if (strAction == QLatin1String("Sell")) {
             qty = -qty;
           }
-          strQuantity = qty.formatMoney("", 2, false);
+          strQuantity = format(qty);
         }
       } else if ((strAction == QLatin1String("Shrsin")) || (strAction == QLatin1String("Shrsout"))) {
         //
@@ -431,14 +414,21 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
         if (strAction == QLatin1String("Shrsout")) {
           qty = -qty;
         }
-        strQuantity = qty.formatMoney("", 2, false);
+        strQuantity = format(qty);
       }
-      strAccName = acc.name();
-      strAccName += m_separator;
+      strAccName = format(acc.name());
       strAction += m_separator;
-      strQuantity += m_separator;
-      strPrice += m_separator;
     }
+
+   if (strAmount.isEmpty())
+      strAmount = m_separator;
+
+   if (strQuantity.isEmpty())
+      strQuantity = m_separator;
+
+    if (strPrice.isEmpty())
+      strPrice = m_separator;
+
     if (strCheckingAccountName.isEmpty()) {
       strCheckingAccountName = m_separator;
     }
@@ -453,3 +443,35 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
   QString date = t.postDate().toString(Qt::ISODate);
   m_map.insertMulti(date, str);
 }
+
+/**
+ * Format string field according to csv rules
+ * @param s string to format
+ * @param withSeparator append field separator to string (default = true)
+ * @return csv formatted string
+ */
+QString CsvWriter::format(const QString &s, bool withSeparator)
+{
+  if (s.isEmpty())
+    return withSeparator ? m_separator : QString();
+  QString m = s;
+  m.remove('\'');
+#if QT_VERSION >= 0x050000
+  m.replace(QLatin1Char('"'), QStringLiteral("\"\""));
+#else
+  m.replace(QLatin1Char('"'), QLatin1String("\"\""));
+#endif
+  return QString("\"%1\"%2").arg(m, withSeparator ? m_separator : QString());
+}
+
+/**
+ * format money value according to csv rules
+ * @param value value to format
+ * @param prec precision used for formatting (default = 2)
+ * @param withSeparator append field separator to string (default = true)
+ * @return formatted value as string
+ */
+QString CsvWriter::format(const MyMoneyMoney &value, int prec, bool withSeparator)
+{
+  return QString("\"%1\"%2").arg(value.formatMoney("", prec, false), withSeparator ? m_separator : QString());
+}
