@@ -41,17 +41,16 @@ public:
   explicit CsvImporterPlugin(QObject* parent, const QVariantList&);
   ~CsvImporterPlugin();
 
-  CSVDialog*        m_csvImporter;
   KAction*          m_action;
 
 public slots:
   bool slotGetStatement(MyMoneyStatement& s);
 
 protected slots:
-  void slotImportFile(void);
+  void slotImportFile();
 
 protected:
-  void createActions(void);
+  void createActions();
   bool importStatement(const MyMoneyStatement& s);
   void processStatement(const MyMoneyStatement& s);
 };
